@@ -15,7 +15,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: tap_reverb.c,v 1.12 2004/01/25 19:16:36 tszilagyi Exp $
+    $Id: tap_reverb.c,v 1.13 2004/06/15 14:50:55 tszilagyi Exp $
 */
 
 
@@ -25,10 +25,47 @@
 #include <math.h>
 
 #include "ladspa.h"
-#include "tap_utils.h"
+
+
+/* ***** VERY IMPORTANT! *****
+ *
+ * If you enable this, the plugin will use float arithmetics in DSP
+ * calculations.  This usually yields lower average CPU usage, but
+ * occasionaly may result in high CPU peaks which cause trouble to you
+ * and your JACK server.  The default is to use fixpoint arithmetics
+ * (with the following #define commented out).  But (depending on the
+ * processor on which you run the code) you may find floating point
+ * mode usable.
+ */
+/* #define REVERB_CALC_FLOAT */
+
+
+
+#ifndef REVERB_CALC_FLOAT
+typedef signed int sample;
+#endif
+
+#ifndef REVERB_CALC_FLOAT
+typedef sample rev_t;
+#else
+typedef LADSPA_Data rev_t;
+#endif
+
+
 #include "tap_reverb_presets.h"
 
 
+
+#ifdef REVERB_CALC_FLOAT
+#define DENORM(x) (((unsigned char)(((*(unsigned int*)&(x))&0x7f800000)>>23))<103)?0.0f:(x)
+#else
+/* coefficient for float to sample (signed int) conversion */
+/* this allows for about 60 dB headroom above 0dB, if 0 dB is equivalent to 1.0f */
+/* As 2^31 equals more than 180 dB, about 120 dB dynamics remains below 0 dB */
+#define F2S 2147483
+#endif
+
+
 /* load plugin data from reverb_data[] into an instance */
 void
 load_plugin_data(LADSPA_Handle Instance) {
@@ -37,7 +74,8 @@ load_plugin_data(LADSPA_Handle Instance) {
 	unsigned long m;
 	int i;
 
-	m = *(ptr->mode);
+
+	m = LIMIT(*(ptr->mode),0,NUM_MODES-1);
 
 	/* load combs data */
 	ptr->num_combs = 2 * reverb_data[m].num_combs;
@@ -47,7 +85,9 @@ load_plugin_data(LADSPA_Handle Instance) {
 		((COMB_FILTER *)(ptr->combs + 2*i))->feedback = 
 			reverb_data[m].combs[i].feedback;
 		((COMB_FILTER *)(ptr->combs + 2*i))->freq_resp =
-			reverb_data[m].combs[i].freq_resp;
+			LIMIT(reverb_data[m].combs[i].freq_resp
+			      * powf(ptr->sample_rate / 44100.0f, 0.8f),
+			      0.0f, 1.0f);
 
 		((COMB_FILTER *)(ptr->combs + 2*i+1))->buflen =
 			((COMB_FILTER *)(ptr->combs + 2*i))->buflen;
@@ -56,18 +96,20 @@ load_plugin_data(LADSPA_Handle Instance) {
 		((COMB_FILTER *)(ptr->combs + 2*i+1))->feedback = 
 			((COMB_FILTER *)(ptr->combs + 2*i))->freq_resp;
 
-
 		/* set initial values: */
 		*(((COMB_FILTER *)(ptr->combs + 2*i))->buffer_pos) = 0;
 		*(((COMB_FILTER *)(ptr->combs + 2*i+1))->buffer_pos) = 0;
-		eq_set_params(((COMB_FILTER *)(ptr->combs + 2*i))->filter,
-			      10000.0f,
-			      reverb_data[m].combs[i].freq_resp * -60.0f,
-			      FREQ_RESP_BWIDTH, ptr->sample_rate);
-		eq_set_params(((COMB_FILTER *)(ptr->combs + 2*i+1))->filter,
-			      10000.0f,
-			      reverb_data[m].combs[i].freq_resp * -60.0f,
-			      FREQ_RESP_BWIDTH, ptr->sample_rate);
+		((COMB_FILTER *)(ptr->combs + 2*i))->last_out = 0;
+		((COMB_FILTER *)(ptr->combs + 2*i+1))->last_out = 0;
+
+		lp_set_params(((COMB_FILTER *)(ptr->combs + 2*i))->filter,
+			      2000.0f + 13000.0f * (1 - reverb_data[m].combs[i].freq_resp)
+			      * ptr->sample_rate / 44100.0f,
+			      BANDPASS_BWIDTH, ptr->sample_rate);
+		lp_set_params(((COMB_FILTER *)(ptr->combs + 2*i+1))->filter,
+			      2000.0f + 13000.0f * (1 - reverb_data[m].combs[i].freq_resp)
+			      * ptr->sample_rate / 44100.0f,
+			      BANDPASS_BWIDTH, ptr->sample_rate);
 	}
 
 	/* load allps data */
@@ -86,6 +128,8 @@ load_plugin_data(LADSPA_Handle Instance) {
 		/* set initial values: */
 		*(((ALLP_FILTER *)(ptr->allps + 2*i))->buffer_pos) = 0;
 		*(((ALLP_FILTER *)(ptr->allps + 2*i+1))->buffer_pos) = 0;
+		((ALLP_FILTER *)(ptr->allps + 2*i))->last_out = 0;
+		((ALLP_FILTER *)(ptr->allps + 2*i+1))->last_out = 0;
 	}
 
 	/* init bandpass filters */
@@ -101,49 +145,45 @@ load_plugin_data(LADSPA_Handle Instance) {
 
 
 
-
-/* push a sample into a ringbuffer and return the sample falling out */
-LADSPA_Data
-push_buffer(LADSPA_Data insample, LADSPA_Data * buffer, 
-	    unsigned long buflen, unsigned long * pos) {
-	
-	LADSPA_Data outsample;
-
-	outsample = buffer[*pos];
-	buffer[(*pos)++] = insample;
-
-	if (*pos >= buflen)
-		*pos = 0;
-	
-	return outsample;
-}
-
-
 /* push a sample into a comb filter and return the sample falling out */
-LADSPA_Data
-comb_run(LADSPA_Data insample, COMB_FILTER * comb) {
+rev_t
+comb_run(rev_t insample, COMB_FILTER * comb) {
 
-	LADSPA_Data outsample;
+	rev_t outsample;
+	rev_t pushin;
 
-	outsample = push_buffer(comb->fb_gain * insample + 
-				biquad_run(comb->filter, comb->fb_gain * comb->last_out),
+	pushin = comb->fb_gain * insample + biquad_run(comb->filter, comb->fb_gain * comb->last_out);
+#ifdef REVERB_CALC_FLOAT
+	pushin = DENORM(pushin);
+#endif
+	outsample = push_buffer(pushin,
 				comb->ringbuffer, comb->buflen, comb->buffer_pos);
-
+#ifdef REVERB_CALC_FLOAT
+	outsample = DENORM(outsample);
+#endif
 	comb->last_out = outsample;
+
 	return outsample;
 }
 
 
 /* push a sample into an allpass filter and return the sample falling out */
-LADSPA_Data
-allp_run(LADSPA_Data insample, ALLP_FILTER * allp) {
+rev_t
+allp_run(rev_t insample, ALLP_FILTER * allp) {
 
-	LADSPA_Data outsample;
-
-	outsample = push_buffer(allp->in_gain * insample + allp->fb_gain * allp->last_out,
+	rev_t outsample;
+	rev_t pushin;
+	pushin = allp->in_gain * allp->fb_gain * insample + allp->fb_gain * allp->last_out;
+#ifdef REVERB_CALC_FLOAT
+	pushin = DENORM(pushin);
+#endif
+	outsample = push_buffer(pushin,
 				allp->ringbuffer, allp->buflen, allp->buffer_pos);
-
+#ifdef REVERB_CALC_FLOAT
+	outsample = DENORM(outsample);
+#endif
 	allp->last_out = outsample;
+
 	return outsample;
 }
 
@@ -155,15 +195,16 @@ comp_coeffs(LADSPA_Handle Instance) {
 	Reverb * ptr = (Reverb *)Instance;
 	int i;
 	
+
 	if (*(ptr->mode) != ptr->old_mode)
 		load_plugin_data(Instance);
 
 	for (i = 0; i < ptr->num_combs / 2; i++) {
 		((COMB_FILTER *)(ptr->combs + 2*i))->fb_gain =
-			powf(0.001f, /* -60 dB */
-			     100000.0f * ((COMB_FILTER *)(ptr->combs + 2*i))->buflen
+			powf(0.001f,
+			     1000.0f * ((COMB_FILTER *)(ptr->combs + 2*i))->buflen
 			     * (1 + FR_R_COMP * ((COMB_FILTER *)(ptr->combs + 2*i))->freq_resp)
-			     / ((COMB_FILTER *)(ptr->combs + 2*i))->feedback
+			     / powf(((COMB_FILTER *)(ptr->combs + 2*i))->feedback/100.0f, 0.89f)
 			     / *(ptr->decay)
 			     / ptr->sample_rate);
 
@@ -189,16 +230,17 @@ comp_coeffs(LADSPA_Handle Instance) {
 
 	for (i = 0; i < ptr->num_allps / 2; i++) {
 		((ALLP_FILTER *)(ptr->allps + 2*i))->fb_gain =
-			powf(0.001f, /* -60 dB */
-			     10000.0f  /* smaller --> longer decay */
+			powf(0.001f, 11000.0f * ((ALLP_FILTER *)(ptr->allps + 2*i))->buflen
+			     / powf(((ALLP_FILTER *)(ptr->allps + 2*i))->feedback/100.0f, 0.88f)
 			     / *(ptr->decay)
-			     / ((ALLP_FILTER *)(ptr->allps + 2*i))->feedback);
+			     / ptr->sample_rate);
 		
 		((ALLP_FILTER *)(ptr->allps + 2*i+1))->fb_gain = 
 			((ALLP_FILTER *)(ptr->allps + 2*i))->fb_gain;
 
-		((ALLP_FILTER *)(ptr->allps + 2*i))->in_gain =
-			-10.0f / ((ALLP_FILTER *)(ptr->allps + 2 * i))->feedback;
+		((ALLP_FILTER *)(ptr->allps + 2*i))->in_gain = -0.06f
+			/ (((ALLP_FILTER *)(ptr->allps + 2 * i))->feedback/100.0f)
+			/ powf((*(ptr->decay) + 3500.0f) / 10000.0f, 1.5f);
 
 		((ALLP_FILTER *)(ptr->allps + 2*i+1))->in_gain = 
 			((ALLP_FILTER *)(ptr->allps + 2*i))->in_gain;
@@ -228,14 +270,55 @@ LADSPA_Handle
 instantiate_Reverb(const LADSPA_Descriptor * Descriptor,
 		   unsigned long             SampleRate) {
 	
-	LADSPA_Handle * ptr;
+	unsigned long i;
+	LADSPA_Handle * p;
+	Reverb * ptr = NULL;
 	
-	if ((ptr = malloc(sizeof(Reverb))) != NULL) {
-		((Reverb *)ptr)->sample_rate = SampleRate;
-		((Reverb *)ptr)->run_adding_gain = 1.0f;
-		return ptr;
+	if ((p = malloc(sizeof(Reverb))) != NULL) {
+		((Reverb *)p)->sample_rate = SampleRate;
+		((Reverb *)p)->run_adding_gain = 1.0f;
+
+		ptr = (Reverb *)p;
+
+		/* allocate memory for comb/allpass filters and other dynamic vars */
+		if ((ptr->combs =
+		     calloc(2 * MAX_COMBS, sizeof(COMB_FILTER))) == NULL)
+			return NULL;
+		for (i = 0; i < 2 * MAX_COMBS; i++) {
+			if ((((COMB_FILTER *)(ptr->combs + i))->ringbuffer =
+			     calloc((unsigned long)MAX_COMB_DELAY * ptr->sample_rate / 1000,
+				    sizeof(LADSPA_Data))) == NULL)
+				return NULL;
+			if ((((COMB_FILTER *)(ptr->combs + i))->buffer_pos =
+			     calloc(1, sizeof(unsigned long))) == NULL)
+				return NULL;
+			if ((((COMB_FILTER *)(ptr->combs + i))->filter =
+			     calloc(1, sizeof(biquad))) == NULL)
+				return NULL;
+		}
+
+		if ((ptr->allps =
+		     calloc(2 * MAX_ALLPS, sizeof(ALLP_FILTER))) == NULL)
+			return NULL;
+		for (i = 0; i < 2 * MAX_ALLPS; i++) {
+			if ((((ALLP_FILTER *)(ptr->allps + i))->ringbuffer =
+			     calloc((unsigned long)MAX_ALLP_DELAY * ptr->sample_rate / 1000,
+				    sizeof(LADSPA_Data))) == NULL)
+				return NULL;
+			if ((((ALLP_FILTER *)(ptr->allps + i))->buffer_pos =
+			     calloc(1, sizeof(unsigned long))) == NULL)
+				return NULL;
+		}
+		
+		if ((ptr->low_pass =
+		     calloc(2, sizeof(biquad))) == NULL)
+			return NULL;
+		if ((ptr->high_pass =
+		     calloc(2, sizeof(biquad))) == NULL)
+			return NULL;
+		
+		return p;
 	}
-	
 	return NULL;
 }
 
@@ -245,82 +328,43 @@ void
 activate_Reverb(LADSPA_Handle Instance) {
 
 	Reverb * ptr = (Reverb *)Instance;
-	int i;
+	unsigned long i,j;
 
-	/* allocate memory for comb/allpass filters and other dynamic vars */
-
-	if ((ptr->combs = calloc(2 * MAX_COMBS, sizeof(COMB_FILTER))) == NULL)
-		exit(1);
 	for (i = 0; i < 2 * MAX_COMBS; i++) {
-		if ((((COMB_FILTER *)(ptr->combs + i))->ringbuffer =
-		     calloc(MAX_COMB_DELAY * ptr->sample_rate / 1000,
-			    sizeof(LADSPA_Data))) == NULL)
-			exit(1);
-		if ((((COMB_FILTER *)(ptr->combs + i))->buffer_pos =
-		     calloc(1, sizeof(unsigned long))) == NULL)
-			exit(1);
-		if ((((COMB_FILTER *)(ptr->combs + i))->filter =
-		     calloc(1, sizeof(biquad))) == NULL)
-			exit(1);
+		for (j = 0; j < (unsigned long)MAX_COMB_DELAY * ptr->sample_rate / 1000; j++)
+		        ((COMB_FILTER *)(ptr->combs + i))->ringbuffer[j] = 0.0f;
+		*(((COMB_FILTER *)(ptr->combs + i))->buffer_pos) = 0;
+		((COMB_FILTER *)(ptr->combs + i))->last_out = 0;
+		biquad_init(((COMB_FILTER *)(ptr->combs + i))->filter);
 	}
 
-	if ((ptr->allps = calloc(2 * MAX_ALLPS, sizeof(ALLP_FILTER))) == NULL)
-		exit(1);
-	for (i = 0; i < 2* MAX_ALLPS; i++) {
-		if ((((ALLP_FILTER *)(ptr->allps + i))->ringbuffer =
-		     calloc(MAX_ALLP_DELAY * ptr->sample_rate / 1000,
-			    sizeof(LADSPA_Data))) == NULL)
-			exit(1);
-		if ((((ALLP_FILTER *)(ptr->allps + i))->buffer_pos =
-		     calloc(1, sizeof(unsigned long))) == NULL)
-			exit(1);
-	}
-
-	if ((ptr->low_pass = calloc(2, sizeof(biquad))) == NULL)
-		exit(1);
-	if ((ptr->high_pass = calloc(2, sizeof(biquad))) == NULL)
-		exit(1);
-
-	load_plugin_data(Instance);
-	comp_coeffs(Instance);
-}
-
-
-/* deactivate a plugin instance */
-void
-deactivate_Reverb(LADSPA_Handle Instance) {
-
-	Reverb * ptr = (Reverb *)Instance;
-	int i;
-
-	/* free memory allocated for comb/allpass filters & co. in activate_Reverb() */
-	
-	for (i = 0; i < 2 * MAX_COMBS; i++) {
-		free(((COMB_FILTER *)(ptr->combs + i))->ringbuffer);
-		free(((COMB_FILTER *)(ptr->combs + i))->buffer_pos);
-		free(((COMB_FILTER *)(ptr->combs + i))->filter);
-	}
 	for (i = 0; i < 2 * MAX_ALLPS; i++) {
-		free(((ALLP_FILTER *)(ptr->allps + i))->ringbuffer);
-		free(((ALLP_FILTER *)(ptr->allps + i))->buffer_pos);
+		for (j = 0; j < (unsigned long)MAX_ALLP_DELAY * ptr->sample_rate / 1000; j++)
+			((ALLP_FILTER *)(ptr->allps + i))->ringbuffer[j] = 0.0f;
+		*(((ALLP_FILTER *)(ptr->allps + i))->buffer_pos) = 0;
+		((ALLP_FILTER *)(ptr->allps + i))->last_out = 0;
 	}
 
-	free(ptr->combs);
-	free(ptr->allps);
-	free(ptr->low_pass);
-	free(ptr->high_pass);
+	biquad_init(ptr->low_pass);
+	biquad_init((biquad *)(ptr->low_pass + 1));
+	biquad_init(ptr->high_pass);
+	biquad_init((biquad *)(ptr->high_pass + 1));
+
+	ptr->old_decay = -10.0f;
+	ptr->old_stereo_enh = -10.0f;
+	ptr->old_mode = -10.0f;
 }
 
 
+
 /* Connect a port to a data location. */
 void 
 connect_port_Reverb(LADSPA_Handle Instance,
 		    unsigned long Port,
 		    LADSPA_Data * DataLocation) {
 	
-	Reverb * ptr;
-	
-	ptr = (Reverb *)Instance;
+	Reverb * ptr = (Reverb *)Instance;
+
 	switch (Port) {
 	case DECAY:
 		ptr->decay = DataLocation;
@@ -372,26 +416,26 @@ run_Reverb(LADSPA_Handle Instance,
 	unsigned long sample_index;
 	int i;
 
-	LADSPA_Data decay = *(ptr->decay);
-	LADSPA_Data drylevel = db2lin(*(ptr->drylevel));
-	LADSPA_Data wetlevel = db2lin(*(ptr->wetlevel));
-	LADSPA_Data combs_en = *(ptr->combs_en);
-	LADSPA_Data allps_en = *(ptr->allps_en);
-	LADSPA_Data bandpass_en = *(ptr->bandpass_en);
-	LADSPA_Data stereo_enh = *(ptr->stereo_enh);
-      	LADSPA_Data mode = *(ptr->mode);
+	LADSPA_Data decay = LIMIT(*(ptr->decay),0.0f,10000.0f);
+	LADSPA_Data drylevel = db2lin(LIMIT(*(ptr->drylevel),-70.0f,10.0f));
+	LADSPA_Data wetlevel = db2lin(LIMIT(*(ptr->wetlevel),-70.0f,10.0f));
+	LADSPA_Data combs_en = LIMIT(*(ptr->combs_en),-2.0f,2.0f);
+	LADSPA_Data allps_en = LIMIT(*(ptr->allps_en),-2.0f,2.0f);
+	LADSPA_Data bandpass_en = LIMIT(*(ptr->bandpass_en),-2.0f,2.0f);
+	LADSPA_Data stereo_enh = LIMIT(*(ptr->stereo_enh),-2.0f,2.0f);
+      	LADSPA_Data mode = LIMIT(*(ptr->mode),0,NUM_MODES-1);
 
 	LADSPA_Data * input_L = ptr->input_L;
 	LADSPA_Data * output_L = ptr->output_L;
 	LADSPA_Data * input_R = ptr->input_R;
 	LADSPA_Data * output_R = ptr->output_R;
 
-	LADSPA_Data out_L = 0;
-	LADSPA_Data out_R = 0;
-	LADSPA_Data in_L = 0;
-	LADSPA_Data in_R = 0;
-	LADSPA_Data combs_out_L = 0;
-	LADSPA_Data combs_out_R = 0;
+	rev_t out_L = 0;
+	rev_t out_R = 0;
+	rev_t in_L = 0;
+	rev_t in_R = 0;
+	rev_t combs_out_L = 0;
+	rev_t combs_out_R = 0;
 
 
 	/* see if the user changed any control since last run */
@@ -409,9 +453,14 @@ run_Reverb(LADSPA_Handle Instance,
 	}
 
 	for (sample_index = 0; sample_index < SampleCount; sample_index++) {
-		
+
+#ifdef REVERB_CALC_FLOAT		
 		in_L = *(input_L++);
 		in_R = *(input_R++);
+#else
+		in_L = (sample)((float)F2S * *(input_L++));
+		in_R = (sample)((float)F2S * *(input_R++));
+#endif
 
 		combs_out_L = in_L;
 		combs_out_R = in_R;
@@ -448,11 +497,17 @@ run_Reverb(LADSPA_Handle Instance,
 				biquad_run(((biquad *)(ptr->high_pass + 1)), combs_out_R);
 		}
 
+#ifdef REVERB_CALC_FLOAT
 		out_L = in_L * drylevel + combs_out_L * wetlevel;
 		out_R = in_R * drylevel + combs_out_R * wetlevel;
-
 		*(output_L++) = out_L;
 		*(output_R++) = out_R;
+#else
+		out_L = (sample)((float)in_L * drylevel + (float)combs_out_L * wetlevel);
+		out_R = (sample)((float)in_R * drylevel + (float)combs_out_R * wetlevel);
+		*(output_L++) = (float)out_L / (float)F2S;
+		*(output_R++) = (float)out_R / (float)F2S;
+#endif
 	}
 }
 
@@ -480,26 +535,26 @@ run_adding_gain_Reverb(LADSPA_Handle Instance,
 	unsigned long sample_index;
 	int i;
 
-	LADSPA_Data decay = *(ptr->decay);
-	LADSPA_Data drylevel = db2lin(*(ptr->drylevel));
-	LADSPA_Data wetlevel = db2lin(*(ptr->wetlevel));
-	LADSPA_Data combs_en = *(ptr->combs_en);
-	LADSPA_Data allps_en = *(ptr->allps_en);
-	LADSPA_Data bandpass_en = *(ptr->bandpass_en);
-	LADSPA_Data stereo_enh = *(ptr->stereo_enh);
-      	LADSPA_Data mode = *(ptr->mode);
+	LADSPA_Data decay = LIMIT(*(ptr->decay),0.0f,10000.0f);
+	LADSPA_Data drylevel = db2lin(LIMIT(*(ptr->drylevel),-70.0f,10.0f));
+	LADSPA_Data wetlevel = db2lin(LIMIT(*(ptr->wetlevel),-70.0f,10.0f));
+	LADSPA_Data combs_en = LIMIT(*(ptr->combs_en),-2.0f,2.0f);
+	LADSPA_Data allps_en = LIMIT(*(ptr->allps_en),-2.0f,2.0f);
+	LADSPA_Data bandpass_en = LIMIT(*(ptr->bandpass_en),-2.0f,2.0f);
+	LADSPA_Data stereo_enh = LIMIT(*(ptr->stereo_enh),-2.0f,2.0f);
+      	LADSPA_Data mode = LIMIT(*(ptr->mode),0,NUM_MODES-1);
 
 	LADSPA_Data * input_L = ptr->input_L;
 	LADSPA_Data * output_L = ptr->output_L;
 	LADSPA_Data * input_R = ptr->input_R;
 	LADSPA_Data * output_R = ptr->output_R;
 
-	LADSPA_Data out_L = 0;
-	LADSPA_Data out_R = 0;
-	LADSPA_Data in_L = 0;
-	LADSPA_Data in_R = 0;
-	LADSPA_Data combs_out_L = 0;
-	LADSPA_Data combs_out_R = 0;
+	rev_t out_L = 0;
+	rev_t out_R = 0;
+	rev_t in_L = 0;
+	rev_t in_R = 0;
+	rev_t combs_out_L = 0;
+	rev_t combs_out_R = 0;
 
 
 	/* see if the user changed any control since last run */
@@ -518,8 +573,13 @@ run_adding_gain_Reverb(LADSPA_Handle Instance,
 
 	for (sample_index = 0; sample_index < SampleCount; sample_index++) {
 		
+#ifdef REVERB_CALC_FLOAT		
 		in_L = *(input_L++);
 		in_R = *(input_R++);
+#else
+		in_L = (sample)((float)F2S * *(input_L++));
+		in_R = (sample)((float)F2S * *(input_R++));
+#endif
 
 		combs_out_L = in_L;
 		combs_out_R = in_R;
@@ -556,11 +616,17 @@ run_adding_gain_Reverb(LADSPA_Handle Instance,
 				biquad_run(((biquad *)(ptr->high_pass + 1)), combs_out_R);
 		}
 
+#ifdef REVERB_CALC_FLOAT		
 		out_L = in_L * drylevel + combs_out_L * wetlevel;
 		out_R = in_R * drylevel + combs_out_R * wetlevel;
-
 		*(output_L++) += out_L * ptr->run_adding_gain;
 		*(output_R++) += out_R * ptr->run_adding_gain;
+#else
+		out_L = (sample)((float)in_L * drylevel + (float)combs_out_L * wetlevel);
+		out_R = (sample)((float)in_R * drylevel + (float)combs_out_R * wetlevel);
+		*(output_L++) += (float)out_L * ptr->run_adding_gain / (float)F2S;
+		*(output_R++) += (float)out_R * ptr->run_adding_gain / (float)F2S;
+#endif
 	}
 }
 
@@ -570,7 +636,27 @@ run_adding_gain_Reverb(LADSPA_Handle Instance,
 void 
 cleanup_Reverb(LADSPA_Handle Instance) {
 
+	int i;
+	Reverb * ptr = (Reverb *)Instance;
+
+	/* free memory allocated for comb/allpass filters & co. in instantiate_Reverb() */
+	for (i = 0; i < 2 * MAX_COMBS; i++) {
+		free(((COMB_FILTER *)(ptr->combs + i))->ringbuffer);
+		free(((COMB_FILTER *)(ptr->combs + i))->buffer_pos);
+		free(((COMB_FILTER *)(ptr->combs + i))->filter);
+	}
+	for (i = 0; i < 2 * MAX_ALLPS; i++) {
+		free(((ALLP_FILTER *)(ptr->allps + i))->ringbuffer);
+		free(((ALLP_FILTER *)(ptr->allps + i))->buffer_pos);
+	}
+
+	free(ptr->combs);
+	free(ptr->allps);
+	free(ptr->low_pass);
+	free(ptr->high_pass);
+
 	free(Instance);
+
 }
 
 
@@ -587,28 +673,7 @@ _init() {
 	char ** port_names;
 	LADSPA_PortDescriptor * port_descriptors;
 	LADSPA_PortRangeHint * port_range_hints;
-	char string[2048]; /* currently used: 1025 out of 2048 */
 
-#ifdef _SHOW_PRESET_NAMES_IN_GUI_
-	int i;
-	char snum[128];
-
-	strcpy(string,"Presets\n");
-	for (i = 0; i < NUM_MODES; i++) {
-		if (i % 2 == 0) {
-			strcat(string, "\n");
-			strcat(string, reverb_data[i].name);
-			sprintf(snum," :%i  ", i);
-			strcat(string, snum);
-		} else {
-			sprintf(snum," %i: ", i);
-			strcat(string, snum);
-			strcat(string, reverb_data[i].name);
-		}
-	}
-#else
-	strcpy(string,"Preset");
-#endif
 	
 	if ((stereo_descriptor =
 	     (LADSPA_Descriptor *)malloc(sizeof(LADSPA_Descriptor))) == NULL)
@@ -657,7 +722,7 @@ _init() {
 	port_names[ALLPS_EN] = strdup("Allpass Filters");
 	port_names[BANDPASS_EN] = strdup("Bandpass Filter");
 	port_names[STEREO_ENH] = strdup("Enhanced Stereo");
-	port_names[MODE] = strdup(string);
+	port_names[MODE] = strdup("Reverb Type");
 	port_names[INPUT_L] = strdup("Input Left");
 	port_names[OUTPUT_L] = strdup("Output Left");
 	port_names[INPUT_R] = strdup("Input Right");
@@ -726,7 +791,7 @@ _init() {
 	stereo_descriptor->run = run_Reverb;
 	stereo_descriptor->run_adding = run_adding_gain_Reverb;
 	stereo_descriptor->set_run_adding_gain = set_run_adding_gain;
-	stereo_descriptor->deactivate = deactivate_Reverb;
+	stereo_descriptor->deactivate = NULL;
 	stereo_descriptor->cleanup = cleanup_Reverb;
 	
 }
